import cv2
import numpy as np
import matplotlib.pyplot as plt


Img = cv2.imread('dog.JPEG')
#Img = cv2.imread('rat.JPEG')
grayImg = cv2.cvtColor(Img, cv2.COLOR_BGR2GRAY)



def binarization_cv2(inputImg ,thresh = 127, maxval = 255):
    thType = cv2.THRESH_OTSU
    '''
    type_examples:
    THRESH_BINARY, THRESH_BINARY_INV, THRESH_TRUNC, THRESH_TOZERO, THRESH_TOZERO_INV,
    THRESH_OTSU, THRESH_TRIANGLE
    '''
    ret, threshImg = cv2.threshold(inputImg, thresh, maxval, thType)
    return threshImg
#outputImg = binarization_cv2(grayImg)

def binarization_otsu(grayImg, minval=0, maxval=255):
    hist = [np.sum(grayImg == i) for i in range(256)]
    s_max = (0,-10)
    for thresh in range(256):
        n1 = sum(hist[:thresh])
        n2 = sum(hist[thresh:])
        if n1 == 0 : mu1 = 0
        else : mu1 = sum([i * hist[i] for i in range(0,thresh)]) / n1   
        if n2 == 0 : mu2 = 0
        else : mu2 = sum([i * hist[i] for i in range(thresh, 256)]) / n2
        s = int(n1) * int(n2) * (int(mu1) - int(mu2)) ** 2
        if s > s_max[1]:
            s_max = (thresh, s)
    t = s_max[0]
    grayImg[grayImg < t] = minval
    grayImg[grayImg >= t] = maxval
    return grayImg
#outputImg = binarization_otsu(grayImg)

def flip_vertical(inputImg):
    row = np.size(inputImg, 0)
    col = np.size(inputImg, 1)
    outputImg = np.zeros((row, col))
    for i in range (0, row):
        for j in range (0, col):
            outputImg[i][j] = inputImg[row-1-i][j]
    return outputImg
#outputImg = flip_vertical(grayImg)

def flip_horizontal(inputImg):
    row = np.size(inputImg, 0)
    col = np.size(inputImg, 1)
    outputImg = np.zeros((row, col))
    for i in range (0, row):
        for j in range (0, col):
            outputImg[i][j] = inputImg[i][col-1-j]
    return outputImg
#outputImg = flip_horizontal(grayImg)

def resize(inputImg, h_out, w_out):
    outputImg = np.empty((h_out, w_out))
    h, w = inputImg.shape[0], inputImg.shape[1]
    ax = w_out / float(w)
    ay = h_out / float(h)
    for j in range(0, h_out):
        for i in range(0, w_out):
            x, y = i/ax, j/ay
            ox, oy = int(x), int(y)
            if ox > w - 2:
                ox = w - 2
            if oy > h - 2:
                oy = h - 2
            dx = x - ox
            dy = y - oy
            outputImg[j][i] = (1 - dx) * (1-dy) * inputImg[oy][ox] + dx * (1-dy) * \
                inputImg[oy][ox+1] + (1-dx) * dy * inputImg[oy][ox+1] + \
                dx * dy * inputImg[oy+1][ox+1]
    return outputImg
#resizeImg = resize(grayImg, grayImg.shape[0]*2, grayImg.shape[1]*2)
#resizeImg = resize(grayImg, int(grayImg.shape[0]/2), int(grayImg.shape[1]/2))

def cut_small(inputImg):
    row = np.size(inputImg, 0)
    col = np.size(inputImg, 1)
    size = min(row, col)
    outputImg = np.zeros((size, size))
    if row < col:
        for i in range (0, size):
            for j in range (0, size):
                outputImg[i][j] = inputImg[i][int((col-row)/2) + j]
    if row > col:
        for i in range (0, size):
            for j in range (0, size):
                outputImg[i][j] = inputImg[int((row-col)/2) + i][j]
    return outputImg
#outputImg = cut_small(grayImg)
    
def cut_big(inputImg):
    row = np.size(inputImg, 0)
    col = np.size(inputImg, 1)
    size = max(row, col)
    outputImg = np.zeros((size, size))
    if row < col:
        for i in range (0, row):
            for j in range (0, size):
                outputImg[int((col-row)/2) + i][j] = inputImg[i][j]
    if row > col:
        for i in range (0, size):
            for j in range (0, col):
                outputImg[i][int((row-col)/2) + j] = inputImg[i][j]
    return outputImg
#outputImg = cut_big(grayImg)
    
def paste_vertical(inputImg):
    row = np.size(inputImg, 0)
    col = np.size(inputImg, 1)
    outputImg = np.zeros((row*2, col))
    for i in range (0, row):
        for j in range (0, col):
            outputImg[i][j] = inputImg[i][j]
            outputImg[i + row][j] = inputImg[i][j]
    return outputImg
#outputImg = paste_vertical(grayImg)
    
def paste_horizontal(inputImg):
    row = np.size(inputImg, 0)
    col = np.size(inputImg, 1)
    outputImg = np.zeros((row, col*2))
    for i in range (0, row):
        for j in range (0, col):
            outputImg[i][j] = inputImg[i][j]
            outputImg[i][j + col] = inputImg[i][j]
    return outputImg
#outputImg = paste_horizontal(grayImg)
    
def histogram(inputImg, filename = 'histogram.JPEG'):
    inputImg = inputImg.flatten()
    n, bins, patches = plt.hist(x=inputImg, bins='auto', color='#0504aa', alpha=0.7, rwidth=0.85)
    plt.grid(axis='y', alpha=0.75)
    plt.xlabel('Value')
    plt.ylabel('Frequency')
    plt.title('histogram')
    maxfreq = n.max()
    plt.ylim(ymax=np.ceil(maxfreq / 10) * 10 if maxfreq % 10 else maxfreq + 10)
    plt.savefig(filename, dpi = 200)
    plt.clf()
#histogram(grayImg)
    

cv2.imwrite('paste_horizontal.JPEG', outputImg)


